GuildRepair = LibStub("AceAddon-3.0"):NewAddon("GuildRepair", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GuildRepair")

local options = {
  name = "GuildRepair",
  handler = GuildRepair,
  type = "group",
  args = {
    maxGoldPerDay = {
      type = "range",
      min = 5,
      max = 500,
      step = 1,
      bigStep = 5,
      name = L["Maximum Gold Per Day"],
      desc = L["The maximum gold allowed for repairs per member per day."],
      order = 1,
      width = "full",
      get = "GetOption",
      set = "SetOption",
    },
    announceStatus = {
      type = "toggle",
      name = L["Announce Changes"],
      desc = L["Toggles the announcement of changes to the raid and officers."],
      order = 3,
      --width = "full",
      get = "GetOption",
      set = "SetOption",
    },
    disableOnLogout = {
      type = "toggle",
      name = L["Disable On Logout"],
      desc = L["Disables guild repairs when you logout."],
      order = 4,
      --width = "full",
      get = "GetOption",
      set = function(i, v) GuildRepair.disableRepairs = value; SetOption(i,v); end,
    },
		ranks = {
			order = 50,
			type = "multiselect",
			name = L["Ranks to Modify"],
			desc = L["Selected ranks will be modified."],
			values = "GetRankTable",
      get = "GetTableOption",
      set = "SetTableOption",
		},
    showDebugMessages = {
      type = "toggle",
      name = L["Show Debug Messages"],
      desc = L["Toggles the display of debug messages in the chat window."],
      order = 100,
      width = "full",
      get = "GetOption",
      set = "SetOption",
    },
  	enable = {
  		type = "execute",
  		name = L["Enable Guild Repairs"],
      desc = L["Enables guild repairs for selected ranks."],
      order = 120,
  		func = "EnableGuildRepairs",
  	},
  	disable = {
  		type = "execute",
  		name = L["Disable Guild Repairs"],
      order = 121,
      desc = L["Disables guild repairs for selected ranks."],
      func = "DisableGuildRepairs",
  	},
  },
}

local defaults = {
  profile = {
  	maxGoldPerDay = 75,
    showDebugMessages = false,
    announceStatus = true,
    disableOnLogout = true,
    ranks = {},
  },
}

-- Global variable that matches self.db.profile.disableOnLogout
-- This is needed because the config database isn't available in 
-- the PLAYER_LOGOUT event.
GuildRepair.disableRepairs = false

-- Register LibDataBroker launcher
--LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("GuildRepair", {
--	type = "launcher",
--	icon = "Interface\\Icons\\Spell_Nature_StormReach",
--	OnClick = function(clickedframe, button)
--		GuildRepair:ToggleGuildRepairs()
--	end,
--})

LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("GuildRepair",
{
	type = "data source",
	icon = "Interface\\ICONS\\Spell_Nature_StormReach",
	text = "GuildRepair",
	OnClick = function(pFrame, pButton) GuildRepair:ToggleGuildRepairs(); end,
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine("GuildRepair",1,1,1)
		tooltip:AddLine(" ")
	  if GuildRepair:AreGuildRepairsEnabled() then
			tooltip:AddLine("Guild repairs are enabled.")
	  else
			tooltip:AddLine("Guild repairs are disabled.")
	  end
		tooltip:AddLine(" ")
		tooltip:AddLine("|cff7fff7fClick|r to toggle guild repairs.")
		tooltip:AddLine(" ")
	end,
});

function GuildRepair:AreGuildRepairsEnabled()
  -- only check the first enabled rank
	for rank = 2, GuildControlGetNumRanks() do 
	  if self.db.profile.ranks[rank-2] then
			GuildControlSetRank(rank)
      -- I know I should use select() here but I don't have time to research it right now.
      -- _, _, _, _, _, _, _, _, _, _, _, _, _, _, withdraw_repair, _, _ = GuildControlGetRankFlags();
      withdraw_repair = select( 15, GuildControlGetRankFlags() )  -- let's hope I can count!
      if withdraw_repair then
        return true
      end
	  end
	end   	
	
	return false
end

function GuildRepair:ToggleGuildRepairs()
  if GuildRepair:AreGuildRepairsEnabled() then
    GuildRepair:DisableGuildRepairs()
  else
    GuildRepair:EnableGuildRepairs()
  end
end

function GuildRepair:OnInitialize()
  -- Called directly after the addon is fully loaded. 
  self.db = LibStub("AceDB-3.0"):New("GuildRepairDB", defaults, "Default")

  LibStub("AceConfig-3.0"):RegisterOptionsTable("GuildRepair", options)
  self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GuildRepair", "GuildRepair")
  self:RegisterChatCommand(L["gr"], "ChatCommand")
  self:RegisterChatCommand(L["guildrepair"], "ChatCommand")
  
  -- initialize the global variable to the value from our profile
  GuildRepair.disableRepairs = self.db.profile.disableOnLogout
end

function GuildRepair:OnEnable()
  -- Called during the PLAYER_LOGIN event, when most of the data provided by the game is already present. 
  
  self:RegisterEvent("GUILD_ROSTER_UPDATE")
  self:RegisterEvent("PLAYER_LOGOUT")
  self:RegisterEvent("RAID_INSTANCE_WELCOME")
  
	--if ( IsInGuild() and GetNumGuildMembers() == 0 ) then
	--	GuildRoster()
	--end
end

function GuildRepair:OnDisable()
  -- Called only when your addon is manually being disabled. 
  self:UnregisterAllEvents()
end

function GuildRepair:PLAYER_LOGOUT()
  -- Sent when the player logs out or the UI is reloaded, just before SavedVariables 
  -- are saved. The event fires after PLAYER_LEAVING_WORLD. 
  if GuildRepair.disableRepairs then
  	GuildRepair:DisableGuildRepairs()
  end
end

function GuildRepair:GUILD_ROSTER_UPDATE()
  self:OutputDebugMessage("GUILD_ROSTER_UPDATE")
end

function GuildRepair:RAID_INSTANCE_WELCOME()
  self:OutputDebugMessage("RAID_INSTANCE_WELCOME")
  GuildRoster()
end

function GuildRepair:GetRankTable()
	local tbl = {}

	for rank = 2, GuildControlGetNumRanks() do 
		tbl[rank-2] = GuildControlGetRankName(rank)
	end   	

	return tbl
end

function GuildRepair:GetOption(info)
	-- self:Print("GetOption("..info[#info]..")")
  return self.db.profile[info[#info]]
end

function GuildRepair:SetOption(info, value)
  self.db.profile[info[#info]] = value
end

function GuildRepair:GetTableOption(info, key)
	-- self:Print("GetTableOption("..info[#info]..", "..key..")")
  return self.db.profile[info[#info]][key]
end

function GuildRepair:SetTableOption(info, key, value)
  self.db.profile[info[#info]][key] = value
end

function GuildRepair:OutputDebugMessage(msg)
  if self.db.profile.showDebugMessages then
		self:Print(msg)
  end
end

function GuildRepair:EnableGuildRepairs()
	self:OutputDebugMessage("Enabling guild repairs...")

	for rank = 2, GuildControlGetNumRanks() do 
	  if self.db.profile.ranks[rank-2] then
			GuildControlSetRank(rank)
			self:OutputDebugMessage("Enabling guild repairs for "..GuildControlGetRankName(rank).."...")
			GuildControlSetRankFlag(15, true)
			SetGuildBankWithdrawLimit(self.db.profile.maxGoldPerDay)
			GuildControlSaveRank(GuildControlGetRankName(rank))
	  end
	end   	

	self:OutputDebugMessage(L["Guild repairs enabled!"])

  if self.db.profile.announceStatus then
	  SendChatMessage(L["Guild repairs enabled!"], "RAID_WARNING")
	  SendChatMessage(L["Guild repairs enabled!"], "OFFICER")	  
  end
end

function GuildRepair:DisableGuildRepairs()
	self:OutputDebugMessage("Disabling guild repairs...")

	for rank = 2, GuildControlGetNumRanks() do 
	  if self.db.profile.ranks[rank-2] then
			GuildControlSetRank(rank)
			self:OutputDebugMessage("Disabling guild repairs for "..GuildControlGetRankName(rank).."...")
			GuildControlSetRankFlag(15, false)
			SetGuildBankWithdrawLimit(0)
			GuildControlSaveRank(GuildControlGetRankName(rank))
		end
	end   	

	self:OutputDebugMessage(L["Guild repairs disabled!"])

  if self.db.profile.announceStatus then
	  SendChatMessage(L["Guild repairs disabled!"], "RAID_WARNING")
	  SendChatMessage(L["Guild repairs disabled!"], "OFFICER")	  
  end
end

function GuildRepair:ChatCommand(input)
  if not input or input:trim() == "" then
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
  else
    LibStub("AceConfigCmd-3.0").HandleCommand(GuildRepair, "gr", "GuildRepair", input)
  end
end
